@@ -1,3 +1,9 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 ''' Bokeh comes with a number of interactive tools.
 
 There are five types of tool interactions:
@@ -13,37 +19,104 @@ There are five types of tool interactions:
 
 For the first three comprise the category of gesture tools, and only
 one tool for each gesture can be active at any given time. The active
-tool is indicated on the toolbar by a highlight next to to the tool.
+tool is indicated on the toolbar by a highlight next to the tool.
 Actions are immediate or modal operations that are only activated when
 their button in the toolbar is pressed. Inspectors are passive tools that
 merely report information or annotate the plot in some way, and may
 always be active regardless of what other tools are currently active.
 
 '''
-from __future__ import absolute_import
 
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
+
+import logging
+log = logging.getLogger(__name__)
+
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
+from types import FunctionType
+
+# External imports
+
+# Bokeh imports
 from ..core.enums import (Anchor, Dimension, Dimensions, Location,
                           TooltipFieldFormatter, TooltipAttachment)
 from ..core.has_props import abstract
 from ..core.properties import (
-    Auto, Bool, Color, Date, Datetime, Dict, Either, Enum, Int, Float,
+    Auto, Bool, Color, Date, Datetime, Dict, Either, Enum, Image, Int, Float,
     Percent, Instance, List, Seq, String, Tuple
 )
+from ..util.compiler import nodejs_compile, CompilationError
+from ..util.dependencies import import_required
+from ..util.future import get_param_info, signature
 from ..core.validation import error
 from ..core.validation.errors import (
     INCOMPATIBLE_BOX_EDIT_RENDERER, INCOMPATIBLE_POINT_DRAW_RENDERER,
     INCOMPATIBLE_POLY_DRAW_RENDERER, INCOMPATIBLE_POLY_EDIT_RENDERER,
-    INCOMPATIBLE_POLY_EDIT_VERTEX_RENDERER
+    INCOMPATIBLE_POLY_EDIT_VERTEX_RENDERER, NO_RANGE_TOOL_RANGES
 )
 from ..model import Model
-from ..util.deprecation import deprecated
 
 from .annotations import BoxAnnotation, PolyAnnotation
 from .callbacks import Callback
 from .glyphs import XYGlyph, Rect, Patches, MultiLine
+from .ranges import Range1d
 from .renderers import Renderer, GlyphRenderer
 from .layouts import LayoutDOM
 
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
+
+__all__ = (
+    'Action',
+    'BoxEditTool',
+    'BoxSelectTool',
+    'BoxZoomTool',
+    'CrosshairTool',
+    'CustomAction',
+    'CustomJSHover',
+    'Drag',
+    'EditTool',
+    'FreehandDrawTool',
+    'HelpTool',
+    'HoverTool',
+    'Inspection',
+    'Gesture',
+    'LassoSelectTool',
+    'PanTool',
+    'PointDrawTool',
+    'PolyDrawTool',
+    'PolyEditTool',
+    'PolySelectTool',
+    'ProxyToolbar',
+    'RangeTool',
+    'RedoTool',
+    'ResetTool',
+    'SaveTool',
+    'Scroll',
+    'Tap',
+    'TapTool',
+    'Tool',
+    'Toolbar',
+    'ToolbarBase',
+    'ToolbarBox',
+    'UndoTool',
+    'WheelPanTool',
+    'WheelZoomTool',
+    'ZoomInTool',
+    'ZoomOutTool',
+)
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
 
 @abstract
 class Tool(Model):
@@ -51,16 +124,6 @@ class Tool(Model):
 
     '''
 
-    @property
-    def plot(self):
-        deprecated("Tool.plot property is no longer needed, and any use deprecated. In the future, accessing Tool.plot will result in an AttributeError")
-        return None
-
-    @plot.setter
-    def plot(self, val):
-        deprecated("Tool.plot property is no longer needed, and any use is deprecated. In the future, accessing Tool.plot will result in an AttributeError")
-        return None
-
 @abstract
 class Action(Tool):
     ''' A base class for tools that are buttons in the toolbar.
@@ -69,35 +132,41 @@ class Action(Tool):
     pass
 
 @abstract
-class Drag(Tool):
+class Gesture(Tool):
     ''' A base class for tools that respond to drag events.
 
     '''
     pass
 
 @abstract
-class Scroll(Tool):
+class Drag(Gesture):
+    ''' A base class for tools that respond to drag events.
+
+    '''
+    pass
+
+@abstract
+class Scroll(Gesture):
     ''' A base class for tools that respond to scroll events.
 
     '''
     pass
 
 @abstract
-class Tap(Tool):
+class Tap(Gesture):
     ''' A base class for tools that respond to tap/click events.
 
     '''
     pass
 
-
 @abstract
-class Inspection(Tool):
+class Inspection(Gesture):
     ''' A base class for tools that perform "inspections", e.g. ``HoverTool``.
 
     '''
     toggleable = Bool(True, help="""
     Whether an on/off toggle button should appear in the toolbar for this
-    inpection tool. If ``False``, the viewers of a plot will not be able to
+    inspection tool. If ``False``, the viewers of a plot will not be able to
     toggle the inspector on or off using the toolbar.
     """)
 
@@ -112,11 +181,15 @@ class ToolbarBase(Model):
     set to None, no logo will be displayed.
     """)
 
+    autohide = Bool(default=False, help="""
+    Whether the toolbar will be hidden by default. Default: False.
+    If True, hides toolbar when cursor is not in canvas.
+    """)
+
     tools = List(Instance(Tool), help="""
     A list of tools to add to the plot.
     """)
 
-
 class Toolbar(ToolbarBase):
     ''' Collect tools to display for a single plot.
 
@@ -139,6 +212,16 @@ class Toolbar(ToolbarBase):
     Specify a tap/click tool to be active when the plot is displayed.
     """)
 
+    active_multi = Instance((Gesture), help="""
+    Specify an active multi-gesture tool, for instance an edit tool or a range
+    tool.
+
+    Note that activating a multi-gesture tool will deactivate any other gesture
+    tools as appropriate. For example, if a pan tool is set as the active drag,
+    and this property is set to a ``BoxEditTool`` instance, the pan tool will
+    be deactivated (i.e. the multi-gesture tool will take precedence).
+    """)
+
 class ProxyToolbar(ToolbarBase):
     ''' A toolbar that allow to merge and proxy tools of toolbars in multiple plots. '''
 
@@ -178,6 +261,73 @@ class PanTool(Drag):
     height of the plot.
     """)
 
+DEFAULT_RANGE_OVERLAY = lambda: BoxAnnotation(
+    level="overlay",
+    render_mode="canvas",
+    fill_color="lightgrey",
+    fill_alpha=0.5,
+    line_color="black",
+    line_alpha=1.0,
+    line_width=0.5,
+    line_dash=[2,2],
+)
+
+class RangeTool(Drag):
+    ''' *toolbar icon*: |range_icon|
+
+    The range tool allows the user to update range objects for either or both
+    of the x- or y-dimensions by dragging a corresponding shaded annotation to
+    move it or change its boundaries.
+
+    A common use case is to add this tool to a plot with a large fixed range,
+    but to configure the tool range from a different plot. When the user
+    manipulates the overlay, the range of the second plot will be updated
+    automatically.
+
+    .. |range_icon| image:: /_images/icons/Range.png
+        :height: 18pt
+
+    '''
+
+    x_range = Instance(Range1d, help="""
+    A range synchronized to the x-dimension of the overlay. If None, the overlay
+    will span the entire x-dimension.
+    """)
+
+    x_interaction = Bool(default=True, help="""
+    Whether to respond to horizontal pan motions when an ``x_range`` is present.
+
+    By default, when an ``x_range`` is specified, it is possible to adjust the
+    horizontal position of the range box by panning horizontally inside the
+    box, or along the top or bottom edge of the box. To disable this, and fix
+    the  range box in place horizontally, set to False. (The box will still
+    update if the ``x_range`` is updated programmatically.)
+    """)
+
+    y_range = Instance(Range1d, help="""
+    A range synchronized to the y-dimension of the overlay. If None, the overlay
+    will span the entire y-dimension.
+    """)
+
+    y_interaction = Bool(default=True, help="""
+    Whether to respond to vertical pan motions when a ``y_range`` is present.
+
+    By default, when a ``y_range`` is specified, it is possible to adjust the
+    vertical position of the range box by panning vertically inside the box, or
+    along the top or bottom edge of the box. To disable this, and fix the range
+    box in place vertically, set to False. (The box will still update if the
+    ``y_range`` is updated programmatically.)
+    """)
+
+    overlay = Instance(BoxAnnotation, default=DEFAULT_RANGE_OVERLAY, help="""
+    A shaded annotation drawn to indicate the configured ranges.
+    """)
+
+    @error(NO_RANGE_TOOL_RANGES)
+    def _check_no_range_tool_ranges(self):
+        if self.x_range is None and self.y_range is None:
+            return "At least one of RangeTool.x_range or RangeTool.y_range must be configured"
+
 class WheelPanTool(Scroll):
     ''' *toolbar icon*: |wheel_pan_icon|
 
@@ -194,7 +344,6 @@ class WheelPanTool(Scroll):
     default the wheel pan tool will pan the plot along the x-axis.
     """)
 
-
 class WheelZoomTool(Scroll):
     ''' *toolbar icon*: |wheel_zoom_icon|
 
@@ -224,6 +373,45 @@ class WheelZoomTool(Scroll):
     zoom out more smoothly, at the cost of losing the focus position.
     """)
 
+    zoom_on_axis = Bool(default=True, help="""
+    Whether scrolling on an axis (outside the central plot area) should
+    zoom that dimension.
+    """)
+
+    speed = Float(default=1/600, help="""
+    Speed at which the wheel zooms. Default is 1/600. Optimal range is between
+    0.001 and 0.09. High values will be clipped. Speed may very between browsers.
+    """)
+
+class CustomAction(Action):
+    ''' Execute a custom action, e.g. ``CustomJS`` callback when a toolbar
+    icon is activated.
+
+    Example:
+
+        .. code-block:: python
+
+            tool = CustomAction(icon="icon.png",
+                                callback=CustomJS(code='alert("foo")'))
+
+            plot.add_tools(tool)
+
+    '''
+
+    action_tooltip = String(default="Perform a Custom Action", help="""
+    Tooltip displayed when hovering over the custom action icon.
+    """)
+
+    callback = Instance(Callback, help="""
+    A Bokeh callback to execute when the custom action icon is activated.
+    """)
+
+    icon = Image(help="""
+    An icon to display in the toolbar.
+
+    The icon can provided as a string filename for an image, a PIL ``Image``
+    object, or an RGB(A) NumPy array.
+    """)
 
 class SaveTool(Action):
     ''' *toolbar icon*: |save_icon|
@@ -240,7 +428,6 @@ class SaveTool(Action):
 
     '''
 
-
 class ResetTool(Action):
     ''' *toolbar icon*: |reset_icon|
 
@@ -258,7 +445,6 @@ class ResetTool(Action):
 
     pass
 
-
 class TapTool(Tap):
     ''' *toolbar icon*: |tap_icon|
 
@@ -284,8 +470,8 @@ class TapTool(Tap):
     have a matching value for their ``name`` attribute will be used.
     """)
 
-    renderers = List(Instance(Renderer), help="""
-    An explicit list of renderers to hit test again. If unset,
+    renderers = Either(Auto, List(Instance(Renderer)), default="auto", help="""
+    An explicit list of renderers to hit test against. If unset,
     defaults to all renderers on a plot.
     """)
 
@@ -293,7 +479,7 @@ class TapTool(Tap):
     This tool can be configured to either make selections or inspections
     on associated data sources. The difference is that selection changes
     propagate across bokeh and other components (e.g. selection glyph)
-    will be notified. Inspecions don't act like this, so it's useful to
+    will be notified. Inspections don't act like this, so it's useful to
     configure `callback` when setting `behavior='inspect'`.
     """)
 
@@ -312,16 +498,20 @@ class TapTool(Tap):
     its ``.source`` attribute and the selection geometry as its
     ``.geometries`` attribute.
 
+    The ``.geometries`` attribute has 5 members.
+    ``.type`` is the geometry type, which always a ``.point`` for a tap event.
+    ``.sx`` and ``.sy`` are the screen X and Y coordinates where the tap occurred.
+    ``.x`` and ``.y`` are the converted data coordinates for the item that has
+    been selected. The ``.x`` and ``.y`` values are based on the axis assigned
+    to that glyph.
+
     .. note::
-        This callback does *not* execute on every tap, only when a glyphs is
+        This callback does *not* execute on every tap, only when a glyph is
         "hit". If you would like to execute a callback on every mouse tap,
         please see :ref:`userguide_interaction_jscallbacks_customjs_interactions`.
 
     """)
 
-
-
-
 class CrosshairTool(Inspection):
     ''' *toolbar icon*: |crosshair_icon|
 
@@ -384,7 +574,7 @@ DEFAULT_BOX_OVERLAY = lambda: BoxAnnotation(
     line_color="black",
     line_alpha=1.0,
     line_width=2,
-    line_dash=[4, 4]
+    line_dash=[4, 4],
 )
 
 class BoxZoomTool(Drag):
@@ -424,6 +614,11 @@ class BoxZoomTool(Drag):
 
     """)
 
+    origin = Enum("corner", "center", default="corner", help="""
+    Indicates whether the rectangular zoom area should originate from a corner
+    (top-left or bottom-right depending on direction) or the center of the box.
+    """)
+
 class ZoomInTool(Action):
     ''' *toolbar icon*: |zoom_in_icon|
 
@@ -467,7 +662,6 @@ class ZoomOutTool(Action):
     Percentage to zoom for each click of the zoom-in tool.
     """)
 
-
 class BoxSelectTool(Drag):
     ''' *toolbar icon*: |box_select_icon|
 
@@ -490,8 +684,8 @@ class BoxSelectTool(Drag):
     have a matching value for their ``name`` attribute will be used.
     """)
 
-    renderers = List(Instance(Renderer), help="""
-    An explicit list of renderers to hit test again. If unset,
+    renderers = Either(Auto, List(Instance(Renderer)), default="auto", help="""
+    An explicit list of renderers to hit test against. If unset,
     defaults to all renderers on a plot.
     """)
 
@@ -512,8 +706,8 @@ class BoxSelectTool(Drag):
 
     callback = Instance(Callback, help="""
     A callback to run in the browser on completion of drawing a selection box.
-    The cb_data parameter that is available to the Callback code will contain
-    one BoxSelectTool-specific field:
+    The ``cb_data`` parameter that is available to the Callback code will contain
+    one ``BoxSelectTool``-specific field:
 
     :geometry: object containing the coordinates of the selection box
     """)
@@ -522,6 +716,11 @@ class BoxSelectTool(Drag):
     A shaded annotation drawn to indicate the selection region.
     """)
 
+    origin = Enum("corner", "center", default="corner", help="""
+    Indicates whether the rectangular selection area should originate from a corner
+    (top-left or bottom-right depending on direction) or the center of the box.
+    """)
+
 DEFAULT_POLY_OVERLAY = lambda: PolyAnnotation(
     level="overlay",
     xs_units="screen",
@@ -561,8 +760,8 @@ class LassoSelectTool(Drag):
     have a matching value for their ``name`` attribute will be used.
     """)
 
-    renderers = List(Instance(Renderer), help="""
-    An explicit list of renderers to hit test again. If unset,
+    renderers = Either(Auto, List(Instance(Renderer)), default="auto", help="""
+    An explicit list of renderers to hit test against. If unset,
     defaults to all renderers on a plot.
     """)
 
@@ -573,8 +772,8 @@ class LassoSelectTool(Drag):
 
     callback = Instance(Callback, help="""
     A callback to run in the browser on every selection of a lasso area.
-    The cb_data parameter that is available to the Callback code will contain
-    one LassoSelectTool-specific field:
+    The ``cb_data`` parameter that is available to the Callback code will contain
+    one ``LassoSelectTool``-specific field:
 
     :geometry: object containing the coordinates of the lasso area
     """)
@@ -583,7 +782,6 @@ class LassoSelectTool(Drag):
     A shaded annotation drawn to indicate the selection region.
     """)
 
-
 class PolySelectTool(Tap):
     ''' *toolbar icon*: |poly_select_icon|
 
@@ -612,14 +810,14 @@ class PolySelectTool(Tap):
     have a matching value for their ``name`` attribute will be used.
     """)
 
-    renderers = List(Instance(Renderer), help="""
-    An explicit list of renderers to hit test again. If unset,
+    renderers = Either(Auto, List(Instance(Renderer)), default="auto", help="""
+    An explicit list of renderers to hit test against. If unset,
     defaults to all renderers on a plot.
     """)
 
     callback = Instance(Callback, help="""
     A callback to run in the browser on completion of drawing a polygon.
-    The cb_data parameter that is available to the Callback code will contain
+    The ``cb_data`` parameter that is available to the Callback code will contain
     one PolySelectTool-specific field:
 
     :geometry: object containing the coordinates of the polygon
@@ -629,6 +827,180 @@ class PolySelectTool(Tap):
     A shaded annotation drawn to indicate the selection region.
     """)
 
+class CustomJSHover(Model):
+    ''' Define a custom formatter to apply to a hover tool field.
+
+    This model can be configured with JavaScript code to format hover tooltips.
+    The JavaScript code has access to the current value to format, some special
+    variables, and any format configured on the tooltip. The variable ``value``
+    will contain the untransformed value. The variable ``special_vars`` will
+    provide a dict with the following contents:
+
+    * ``x`` data-space x-coordinate of the mouse
+    * ``y`` data-space y-coordinate of the mouse
+    * ``sx`` screen-space x-coordinate of the mouse
+    * ``sy`` screen-space y-coordinate of the mouse
+    * ``data_x`` data-space x-coordinate of the hovered glyph
+    * ``data_y`` data-space y-coordinate of the hovered glyph
+    * ``indices`` column indices of all currently hovered glyphs
+    * ``name`` value of the ``name`` property of the hovered glyph renderer
+
+    If the hover is over a "multi" glyph such as ``Patches`` or ``MultiLine``
+    then a ``segment_index`` key will also be present.
+
+    Finally, the value of the format passed in the tooltip specification is
+    available as the ``format`` variable.
+
+    Example:
+
+        As an example, the following code adds a custom formatter to format
+        WebMercator northing coordinates (in meters) as a latitude:
+
+        .. code-block:: python
+
+            lat_custom = CustomJSHover(code="""
+                var projections = require("core/util/projections");
+                var x = special_vars.x
+                var y = special_vars.y
+                var coords = projections.wgs84_mercator.inverse([x, y])
+                return "" + coords[1]
+            """)
+
+            p.add_tools(HoverTool(
+                tooltips=[( 'lat','@y{custom}' )],
+                formatters=dict(y=lat_custom)
+            ))
+
+    .. warning::
+        The explicit purpose of this Bokeh Model is to embed *raw JavaScript
+        code* for a browser to execute. If any part of the code is derived
+        from untrusted user inputs, then you must take appropriate care to
+        sanitize the user input prior to passing to Bokeh.
+
+    '''
+
+    @classmethod
+    def from_py_func(cls, code):
+        ''' Create a ``CustomJSHover`` instance from a Python functions. The
+        function is translated to JavaScript using PScript.
+
+        The python functions must have no positional arguments. It's
+        possible to pass Bokeh models (e.g. a ``ColumnDataSource``) as keyword
+        arguments to the functions.
+
+        The ``code`` function namespace will contain the variable ``value``
+        (the untransformed value) at render time as well as ``format`` and
+        ``special_vars`` as described in the class description.
+
+        Args:
+            code (function) : a scalar function to transform a single ``value``
+
+        Returns:
+            CustomJSHover
+
+        '''
+        from bokeh.util.deprecation import deprecated
+        deprecated("'from_py_func' is deprecated and will be removed in an eventual 2.0 release. "
+                   "Use CustomJSHover directly instead.")
+
+        if not isinstance(code, FunctionType):
+            raise ValueError('CustomJSHover.from_py_func only accepts function objects.')
+
+        pscript = import_required('pscript',
+                                  'To use Python functions for CustomJSHover, you need PScript ' +
+                                  '("conda install -c conda-forge pscript" or "pip install pscript")')
+
+        def pscript_compile(code):
+            sig = signature(code)
+
+            all_names, default_values = get_param_info(sig)
+
+            if len(all_names) - len(default_values) != 0:
+                raise ValueError("Function may only contain keyword arguments.")
+
+            if default_values and not any(isinstance(value, Model) for value in default_values):
+                raise ValueError("Default value must be a Bokeh Model.")
+
+            func_kwargs = dict(zip(all_names, default_values))
+
+            # Wrap the code attr in a function named `code` and call it
+            # with arguments that match the `args` attr
+            code = pscript.py2js(code, 'transformer') + 'return transformer(%s);\n' % ', '.join(all_names)
+            return code, func_kwargs
+
+        jsfunc, func_kwargs = pscript_compile(code)
+
+        return cls(code=jsfunc, args=func_kwargs)
+
+    @classmethod
+    def from_coffeescript(cls, code, args={}):
+        ''' Create a CustomJSHover instance from a CoffeeScript snippet.
+        The function bodies are translated to JavaScript functions using
+        node and therefore require return statements.
+
+        The ``code`` snippet namespace will contain the variable ``value``
+        (the untransformed value) at render time as well as ``format`` and
+        ``special_vars`` as described in the class description.
+
+        Example:
+
+        .. code-block:: coffeescript
+
+            formatter = CustomJSHover.from_coffeescript("return value + " total")
+
+        Args:
+            code (str) :
+                A coffeescript snippet to transform a single ``value`` value
+
+        Returns:
+            CustomJSHover
+
+        '''
+        compiled = nodejs_compile(code, lang="coffeescript", file="???")
+        if "error" in compiled:
+            raise CompilationError(compiled.error)
+
+        return cls(code=compiled.code, args=args)
+
+    args = Dict(String, Instance(Model), help="""
+    A mapping of names to Bokeh plot objects. These objects are made
+    available to the callback code snippet as the values of named
+    parameters to the callback.
+    """)
+
+    code = String(default="", help="""
+    A snippet of JavaScript code to transform a single value. The variable
+    ``value`` will contain the untransformed value and can be expected to be
+    present in the function namespace at render time. Additionally, the
+    variable ``special_vars`` will be available, and will provide a dict
+    with the following contents:
+
+    * ``x`` data-space x-coordinate of the mouse
+    * ``y`` data-space y-coordinate of the mouse
+    * ``sx`` screen-space x-coordinate of the mouse
+    * ``sy`` screen-space y-coordinate of the mouse
+    * ``data_x`` data-space x-coordinate of the hovered glyph
+    * ``data_y`` data-space y-coordinate of the hovered glyph
+    * ``indices`` column indices of all currently hovered glyphs
+
+    If the hover is over a "multi" glyph such as ``Patches`` or ``MultiLine``
+    then a ``segment_index`` key will also be present.
+
+    Finally, the value of the format passed in the tooltip specification is
+    available as the ``format`` variable.
+
+    The snippet will be made into the body of a function and therefore requires
+    a return statement.
+
+    Example:
+
+        .. code-block:: javascript
+
+            code = '''
+            return value + " total"
+            '''
+    """)
+
 class HoverTool(Inspection):
     ''' *toolbar icon*: |crosshair_icon|
 
@@ -657,7 +1029,7 @@ class HoverTool(Inspection):
             ("total", "@total{$0,0.00}"
         ]
 
-    You can also supply a ``Callback`` to the HoverTool, to build custom
+    You can also supply a ``Callback`` to the ``HoverTool``, to build custom
     interactions on hover. In this case you may want to turn the tooltips
     off by setting ``tooltips=None``.
 
@@ -695,15 +1067,15 @@ class HoverTool(Inspection):
     have a matching value for their ``name`` attribute will be used.
     """)
 
-    renderers = List(Instance(Renderer), help="""
-    An explicit list of renderers to hit test again. If unset,
+    renderers = Either(Auto, List(Instance(Renderer)), default="auto", help="""
+    An explicit list of renderers to hit test against. If unset,
     defaults to all renderers on a plot.
     """)
 
     callback = Instance(Callback, help="""
     A callback to run in the browser whenever the input's value changes. The
-    cb_data parameter that is available to the Callback code will contain two
-    HoverTool specific fields:
+    ``cb_data`` parameter that is available to the Callback code will contain two
+    ``HoverTool`` specific fields:
 
     :index: object containing the indices of the hovered points in the data source
     :geometry: object containing the coordinates of the hover cursor
@@ -724,7 +1096,8 @@ class HoverTool(Inspection):
 
     Field names starting with "$" are special, known fields:
 
-    :$index: index of selected point in the data source
+    :$index: index of hovered point in the data source
+    :$name: value of the ``name`` property of the hovered glyph renderer
     :$x: x-coordinate under the cursor in data space
     :$y: y-coordinate under the cursor in data space
     :$sx: x-coordinate under the cursor in screen (canvas) space
@@ -744,6 +1117,13 @@ class HoverTool(Inspection):
     surrounding it in curly braces, e.g. ``@{adjusted close}`` will display
     values from a column named ``"adjusted close"``.
 
+    Sometimes (especially with stacked charts) it is desirable to allow the
+    name of the column be specified indirectly. The field name ``@$name`` is
+    distinguished in that it will look up the ``name`` field on the hovered
+    glyph renderer, and use that value as the column name. For instance, if
+    a user hovers with the name ``"US East"``, then ``@$name`` is equivalent to
+    ``@{US East}``.
+
     By default, values for fields (e.g. ``@foo``) are displayed in a basic
     numeric format. However it is possible to control the formatting of values
     more precisely. Fields can be modified by appending a format specified to
@@ -773,7 +1153,7 @@ class HoverTool(Inspection):
 
     """).accepts(Dict(String, String), lambda d: list(d.items()))
 
-    formatters = Dict(String, Enum(TooltipFieldFormatter), default=lambda: dict(), help="""
+    formatters = Dict(String, Either(Enum(TooltipFieldFormatter), Instance(CustomJSHover)), default=lambda: dict(), help="""
     Specify the formatting scheme for data source columns, e.g.
 
     .. code-block:: python
@@ -781,7 +1161,7 @@ class HoverTool(Inspection):
         tool.formatters = dict(date="datetime")
 
     will cause format specifications for the "date" column to be interpreted
-    according to the "datetime" formatting scheme. The following schemed are
+    according to the "datetime" formatting scheme. The following schemes are
     available:
 
     :``"numeral"``:
@@ -837,7 +1217,7 @@ class HoverTool(Inspection):
     """)
 
     show_arrow = Bool(default=True, help="""
-    Whether tooltip's arrow should be showed.
+    Whether tooltip's arrow should be shown.
     """)
 
 DEFAULT_HELP_TIP = "Click the question mark to learn more about Bokeh plot tools."
@@ -880,19 +1260,30 @@ class RedoTool(Action):
     '''
 
 @abstract
-class EditTool(Tool):
+class EditTool(Gesture):
     ''' A base class for all interactive draw tool types.
 
     '''
 
+    custom_tooltip = String(None, help="""
+    A custom tooltip label to override the default name.
+    """)
+
     empty_value = Either(Bool, Int, Float, Date, Datetime, Color, help="""
     Defines the value to insert on non-coordinate columns when a new
-    glyph is inserted into the ColumnDataSource columns, e.g. when a
+    glyph is inserted into the ``ColumnDataSource`` columns, e.g. when a
     circle glyph defines 'x', 'y' and 'color' columns, adding a new
     point will add the x and y-coordinates to 'x' and 'y' columns and
     the color column will be filled with the defined empty value.
     """)
 
+    custom_icon = Image(help="""
+    An icon to display in the toolbar.
+
+    The icon can provided as a string filename for an image, a PIL ``Image``
+    object, or an RGB(A) NumPy array.
+    """)
+
     renderers = List(Instance(Renderer), help="""
     An explicit list of renderers corresponding to scatter glyphs
     that may be edited.
@@ -901,12 +1292,11 @@ class EditTool(Tool):
 class BoxEditTool(EditTool, Drag, Tap):
     ''' *toolbar icon*: |box_edit_icon|
 
-    The BoxEditTool allows drawing, dragging and deleting ``Rect``
-    glyphs on one or more renderers by editing the underlying
-    ``ColumnDataSource`` data. Like other drawing tools, the renderers
-    that are to be edited must be supplied explicitly as a list. When
-    drawing a new box the data will always be added to the
-    ``ColumnDataSource`` on the first supplied renderer.
+    Allows drawing, dragging and deleting ``Rect`` glyphs on one or more
+    renderers by editing the underlying ``ColumnDataSource`` data. Like other
+    drawing tools, the renderers that are to be edited must be supplied
+    explicitly as a list. When drawing a new box the data will always be added
+    to the ``ColumnDataSource`` on the first supplied renderer.
 
     The tool will automatically modify the columns on the data source
     corresponding to the ``x``, ``y``, ``width`` and ``height`` values
@@ -949,6 +1339,13 @@ class BoxEditTool(EditTool, Drag, Tap):
     the vertical dimension can be controlled.
     """)
 
+    num_objects = Int(default=0, help="""
+    Defines a limit on the number of boxes that can be drawn. By
+    default there is no limit on the number of objects, but if enabled
+    the oldest drawn box will be dropped to make space for the new box
+    being added.
+    """)
+
     @error(INCOMPATIBLE_BOX_EDIT_RENDERER)
     def _check_compatible_renderers(self):
         incompatible_renderers = []
@@ -956,23 +1353,29 @@ class BoxEditTool(EditTool, Drag, Tap):
             if not isinstance(renderer.glyph, Rect):
                 incompatible_renderers.append(renderer)
         if incompatible_renderers:
-            glyph_types = ', '.join([type(renderer.glyph).__name__ for renderer in incompatible_renderers])
+            glyph_types = ', '.join(type(renderer.glyph).__name__ for renderer in incompatible_renderers)
             return "%s glyph type(s) found." % glyph_types
 
 class PointDrawTool(EditTool, Drag, Tap):
     ''' *toolbar icon*: |point_draw_icon|
 
     The PointDrawTool allows adding, dragging and deleting point-like
-    glyphs (of ``XYGlyph`` type) on one or more renderers by editing
-    the underlying ``ColumnDataSource`` data. Like other drawing
-    tools, the renderers that are to be edited must be supplied
-    explicitly as a list. Any newly added points will be inserted on
-    the ``ColumnDataSource`` of the first supplied renderer.
+    glyphs (of ``XYGlyph`` type) on one or more renderers by editing the
+    underlying ``ColumnDataSource`` data. Like other drawing tools, the
+    renderers that are to be edited must be supplied explicitly as a list.
+    Any newly added points will be inserted on the ``ColumnDataSource`` of
+    the first supplied renderer.
 
     The tool will automatically modify the columns on the data source
-    corresponding to the ``x`` and ``y`` values of the glyph. Any
-    additional columns in the data source will be padded with the
-    declared ``empty_value``, when adding a new point.
+    corresponding to the ``x`` and ``y`` values of the glyph. Any additional
+    columns in the data source will be padded with the given ``empty_value``
+    when adding a new point.
+
+    .. note::
+        The data source updates will trigger data change events continuously
+        throughout the edit operations on the BokehJS side. In Bokeh server
+        apps, the data source will only be synced once, when the edit operation
+        finishes.
 
     The supported actions include:
 
@@ -994,6 +1397,13 @@ class PointDrawTool(EditTool, Drag, Tap):
     drag = Bool(default=True, help="""
     Enables dragging of existing points on pan events.""")
 
+    num_objects = Int(default=0, help="""
+    Defines a limit on the number of points that can be drawn. By
+    default there is no limit on the number of objects, but if enabled
+    the oldest drawn point will be dropped to make space for the new
+    point.
+    """)
+
     @error(INCOMPATIBLE_POINT_DRAW_RENDERER)
     def _check_compatible_renderers(self):
         incompatible_renderers = []
@@ -1001,7 +1411,7 @@ class PointDrawTool(EditTool, Drag, Tap):
             if not isinstance(renderer.glyph, XYGlyph):
                 incompatible_renderers.append(renderer)
         if incompatible_renderers:
-            glyph_types = ', '.join([type(renderer.glyph).__name__ for renderer in incompatible_renderers])
+            glyph_types = ', '.join(type(renderer.glyph).__name__ for renderer in incompatible_renderers)
             return "%s glyph type(s) found." % glyph_types
 
 class PolyDrawTool(EditTool, Drag, Tap):
@@ -1009,7 +1419,7 @@ class PolyDrawTool(EditTool, Drag, Tap):
 
     The PolyDrawTool allows drawing, selecting and deleting
     ``Patches`` and ``MultiLine`` glyphs on one or more renderers by
-    editing the underlying ColumnDataSource data. Like other drawing
+    editing the underlying ``ColumnDataSource`` data. Like other drawing
     tools, the renderers that are to be edited must be supplied
     explicitly as a list.
 
@@ -1018,6 +1428,11 @@ class PolyDrawTool(EditTool, Drag, Tap):
     additional columns in the data source will be padded with the
     declared ``empty_value``, when adding a new point.
 
+    If a ``vertex_renderer`` with an point-like glyph is supplied the
+    ``PolyDrawTool`` will use it to display the vertices of the
+    multi-lines/patches on all supplied renderers. This also enables
+    the ability to snap to existing vertices while drawing.
+
     The supported actions include:
 
     * Add patch/multi-line: Double tap to add the first vertex, then
@@ -1040,6 +1455,17 @@ class PolyDrawTool(EditTool, Drag, Tap):
     drag = Bool(default=True, help="""
     Enables dragging of existing patches and multi-lines on pan events.""")
 
+    num_objects = Int(default=0, help="""
+    Defines a limit on the number of patches or multi-lines that can
+    be drawn. By default there is no limit on the number of objects,
+    but if enabled the oldest drawn patch or multi-line will be
+    dropped to make space for the new patch or multi-line.
+    """)
+
+    vertex_renderer = Instance(GlyphRenderer, help="""
+    The renderer used to render the vertices of a selected line or
+    polygon.""")
+
     @error(INCOMPATIBLE_POLY_DRAW_RENDERER)
     def _check_compatible_renderers(self):
         incompatible_renderers = []
@@ -1047,9 +1473,56 @@ class PolyDrawTool(EditTool, Drag, Tap):
             if not isinstance(renderer.glyph, (MultiLine, Patches)):
                 incompatible_renderers.append(renderer)
         if incompatible_renderers:
-            glyph_types = ', '.join([type(renderer.glyph).__name__ for renderer in incompatible_renderers])
+            glyph_types = ', '.join(type(renderer.glyph).__name__ for renderer in incompatible_renderers)
             return "%s glyph type(s) found." % glyph_types
 
+    @error(INCOMPATIBLE_POLY_EDIT_VERTEX_RENDERER)
+    def _check_compatible_vertex_renderer(self):
+        if self.vertex_renderer is None:
+            return
+        glyph = self.vertex_renderer.glyph
+        if not isinstance(glyph, XYGlyph):
+            return "glyph type %s found." % type(glyph).__name__
+
+class FreehandDrawTool(EditTool, Drag, Tap):
+    ''' *toolbar icon*: |freehand_draw_icon|
+
+    Allows freehand drawing of ``Patches`` and ``MultiLine`` glyphs. The glyph
+    to draw may be defined via the ``renderers`` property.
+
+    The tool will automatically modify the columns on the data source
+    corresponding to the ``xs`` and ``ys`` values of the glyph. Any
+    additional columns in the data source will be padded with the
+    declared ``empty_value``, when adding a new point.
+
+    The supported actions include:
+
+    * Draw vertices: Click and drag to draw a line
+
+    * Delete patch/multi-line: Tap a patch/multi-line to select it
+      then press <<backspace>> key while the mouse is within the plot
+      area.
+
+    .. |freehand_draw_icon| image:: /_images/icons/FreehandDraw.png
+        :height: 18pt
+    '''
+
+    num_objects = Int(default=0, help="""
+    Defines a limit on the number of patches or multi-lines that can
+    be drawn. By default there is no limit on the number of objects,
+    but if enabled the oldest drawn patch or multi-line will be
+    overwritten when the limit is reached.
+    """)
+
+    @error(INCOMPATIBLE_POLY_DRAW_RENDERER)
+    def _check_compatible_renderers(self):
+        incompatible_renderers = []
+        for renderer in self.renderers:
+            if not isinstance(renderer.glyph, (MultiLine, Patches)):
+                incompatible_renderers.append(renderer)
+        if incompatible_renderers:
+            glyph_types = ', '.join(type(renderer.glyph).__name__ for renderer in incompatible_renderers)
+            return "%s glyph type(s) found." % glyph_types
 
 class PolyEditTool(EditTool, Drag, Tap):
     ''' *toolbar icon*: |poly_edit_icon|
@@ -1101,6 +1574,18 @@ class PolyEditTool(EditTool, Drag, Tap):
             if not isinstance(renderer.glyph, (MultiLine, Patches)):
                 incompatible_renderers.append(renderer)
         if incompatible_renderers:
-            glyph_types = ', '.join([type(renderer.glyph).__name__
-                                     for renderer in incompatible_renderers])
+            glyph_types = ', '.join(type(renderer.glyph).__name__
+                                    for renderer in incompatible_renderers)
             return "%s glyph type(s) found." % glyph_types
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
