@@ -1,3 +1,9 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 ''' The resources module provides the Resources class for easily configuring
 how BokehJS code and CSS resources should be located, loaded, and embedded in
 Bokeh documents.
@@ -10,195 +16,52 @@ Attributes:
 
 '''
 
-from __future__ import absolute_import
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 import logging
-logger = logging.getLogger(__name__)
+log = logging.getLogger(__name__)
 
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
+import re
 import json
 from os.path import basename, join, relpath
-import re
 
+# External imports
 from six import string_types
 
+# Bokeh imports
 from . import __version__
 from .core.templates import JS_RESOURCES, CSS_RESOURCES
+from .model import Model
 from .settings import settings
 
 from .util.paths import bokehjsdir
-from .util.string import snakify
 from .util.session_id import generate_session_id
-from .model import Model
+
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
 
 DEFAULT_SERVER_HOST = "localhost"
 DEFAULT_SERVER_PORT = 5006
 DEFAULT_SERVER_HTTP_URL = "http://%s:%d/" % (DEFAULT_SERVER_HOST, DEFAULT_SERVER_PORT)
 
-def websocket_url_for_server_url(url):
-    if url.startswith("http:"):
-        reprotocoled = "ws" + url[4:]
-    elif url.startswith("https:"):
-        reprotocoled = "wss" + url[5:]
-    else:
-        raise ValueError("URL has unknown protocol " + url)
-    if reprotocoled.endswith("/"):
-        return reprotocoled + "ws"
-    else:
-        return reprotocoled + "/ws"
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
 
+# __all__ defined at the bottom on the class module
 
-def server_url_for_websocket_url(url):
-    if url.startswith("ws:"):
-        reprotocoled = "http" + url[2:]
-    elif url.startswith("wss:"):
-        reprotocoled = "https" + url[3:]
-    else:
-        raise ValueError("URL has non-websocket protocol " + url)
-    if not reprotocoled.endswith("/ws"):
-        raise ValueError("websocket URL does not end in /ws")
-    return reprotocoled[:-2]
-
-class _SessionCoordinates(object):
-    """ Internal class used to parse kwargs for server URL, app_path, and session_id."""
-    def __init__(self, kwargs):
-        """ Using kwargs which may have extra stuff we don't care about, compute websocket url and session ID."""
-
-        self._base_url = kwargs.get('url', DEFAULT_SERVER_HTTP_URL)
-        if self._base_url is None:
-            raise ValueError("url cannot be None")
-        if self._base_url == 'default':
-            self._base_url = DEFAULT_SERVER_HTTP_URL
-        if self._base_url.startswith("ws"):
-            raise ValueError("url should be the http or https URL for the server, not the websocket URL")
-
-        # base_url always has trailing slash, host:port/{prefix/}
-        if not self._base_url.endswith("/"):
-            self._base_url = self._base_url + "/"
-
-        self._app_path = kwargs.get('app_path', '/')
-        if self._app_path is None:
-            raise ValueError("app_path cannot be None")
-        if not self._app_path.startswith("/"):
-            raise ValueError("app_path should start with a '/' character")
-        if self._app_path != '/' and self._app_path.endswith("/"):
-            self._app_path = self._app_path[:-1] # chop off trailing slash
-
-        self._session_id = kwargs.get('session_id')
-        # we lazy-generate the session_id so we can generate
-        # it server-side when appropriate
-
-        # server_url never has trailing slash because it's
-        # prettier like host:port/app_path without a slash
-        if self._app_path == '/':
-            self._server_url = self._base_url[:-1] # chop off trailing slash
-        else:
-            self._server_url = self._base_url + self._app_path[1:]
-
-    @property
-    def websocket_url(self):
-        """ Websocket URL derived from the kwargs provided."""
-        return websocket_url_for_server_url(self._server_url)
-
-    @property
-    def server_url(self):
-        """ Server URL including app path derived from the kwargs provided."""
-        return self._server_url
-
-    @property
-    def url(self):
-        """ Server base URL derived from the kwargs provided (no app path)."""
-        return self._base_url
-
-    @property
-    def session_id(self):
-        """ Session ID derived from the kwargs provided."""
-        if self._session_id is None:
-            self._session_id = generate_session_id()
-        return self._session_id
-
-    @property
-    def session_id_allowing_none(self):
-        """ Session ID provided in kwargs, keeping it None if it hasn't been generated yet.
-
-        The purpose of this is to preserve ``None`` as long as possible... in some cases
-        we may never generate the session ID because we generate it on the server.
-        """
-        return self._session_id
-
-    @property
-    def app_path(self):
-        """ App path derived from the kwargs provided."""
-        return self._app_path
-
-DEFAULT_SERVER_WEBSOCKET_URL = websocket_url_for_server_url(DEFAULT_SERVER_HTTP_URL)
-
-_DEV_PAT = re.compile(r"^(\d)+\.(\d)+\.(\d)+(dev|rc)")
-
-
-def _cdn_base_url():
-    return "https://cdn.pydata.org"
-
-# XXX: this shouldn't be here, however we mix classes and global functions and
-# we end up with code like this. This module needs a redesign and rewrite soon.
-_component_filter = {
-    'js' : [],
-    'css': ['bokeh-compiler'],
-}
-
-def _get_cdn_urls(components, version=None, minified=True):
-    if version is None:
-        if settings.docs_cdn():
-            version = settings.docs_cdn()
-        else:
-            version = __version__.split('-')[0]
-
-    # check if we want minified js and css
-    _min = ".min" if minified else ""
-
-    base_url = _cdn_base_url()
-    dev_container = 'bokeh/dev'
-    rel_container = 'bokeh/release'
-
-    # check the 'dev' fingerprint
-    container = dev_container if _DEV_PAT.match(version) else rel_container
-
-    if version.endswith(('dev', 'rc')):
-        logger.debug("Getting CDN URL for local dev version will not produce usable URL")
-
-    def mk_url(comp, kind):
-        return '%s/%s/%s-%s%s.%s' % (base_url, container, comp, version, _min, kind)
-
-    result = {
-        'urls'     : lambda kind: [ mk_url(component, kind) \
-            for component in components if component not in _component_filter[kind] ],
-        'messages' : [],
-    }
-
-    if len(__version__.split('-')) > 1:
-        result['messages'].append({
-            "type" : "warn",
-            "text" : ("Requesting CDN BokehJS version '%s' from Bokeh development version '%s'. "
-                      "This configuration is unsupported and may not work!" % (version, __version__))
-        })
-
-    return result
-
-
-def _get_server_urls(components, root_url, minified=True, path_versioner=None):
-    _min = ".min" if minified else ""
-
-    def mk_url(comp, kind):
-        path = "%s/%s%s.%s" % (kind, comp, _min, kind)
-        if path_versioner is not None:
-            path = path_versioner(path)
-        return '%sstatic/%s' % (root_url, path)
-
-    return {
-        'urls'     : lambda kind: [ mk_url(component, kind) \
-            for component in components if component not in _component_filter[kind] ],
-        'messages' : [],
-    }
-
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
 
 class BaseResources(object):
     _default_root_dir = "."
@@ -208,8 +71,12 @@ class BaseResources(object):
                  minified=True, log_level="info", root_url=None,
                  path_versioner=None, components=None):
 
-        self.components = components if components is not None \
-            else ["bokeh", "bokeh-widgets", "bokeh-compiler"]
+        self._components = components
+
+        if hasattr(self, '_js_components'):
+            self.js_components = self._js_components
+        if hasattr(self, '_css_components'):
+            self.css_components = self._css_components
 
         self.mode = settings.resources(mode);           del mode
         self.root_dir = settings.rootdir(root_dir);     del root_dir
@@ -219,7 +86,7 @@ class BaseResources(object):
         self.path_versioner = path_versioner;           del path_versioner
 
         if root_url and not root_url.endswith("/"):
-            logger.warning("root_url should end with a /, adding one")
+            log.warning("root_url should end with a /, adding one")
             root_url = root_url + "/"
         self._root_url = root_url
         if self.mode not in ['inline', 'cdn', 'server', 'server-dev', 'relative', 'relative-dev', 'absolute', 'absolute-dev']:
@@ -248,6 +115,8 @@ class BaseResources(object):
             server = self._server_urls()
             self.messages.extend(server['messages'])
 
+    # Properties --------------------------------------------------------------
+
     @property
     def log_level(self):
         return self._log_level
@@ -258,21 +127,28 @@ class BaseResources(object):
             "trace", "debug", "info", "warn", "error", "fatal"
         ]
         if not (level is None or level in valid_levels):
-            raise ValueError("Unknown log level '%s', valid levels are: %s", str(valid_levels))
+            raise ValueError("Unknown log level '{}', valid levels are: {}".format(level, str(valid_levels)))
         self._log_level = level
 
     @property
     def root_url(self):
-        if self._root_url:
+        if self._root_url is not None:
             return self._root_url
         else:
             return self._default_root_url
 
+    # Public methods ----------------------------------------------------------
+
+    def components(self, kind):
+        components = self.js_components if kind == 'js' else self.css_components
+        if self._components is not None:
+            components = [ c for c in components if c in self._components ]
+        return components
+
     def _file_paths(self, kind):
         bokehjs_dir = bokehjsdir(self.dev)
         minified = ".min" if not self.dev and self.minified else ""
-        files = [ "%s%s.%s" % (component, minified, kind) \
-            for component in self.components if component not in _component_filter[kind] ]
+        files = [ "%s%s.%s" % (component, minified, kind) for component in self.components(kind) ]
         paths = [ join(bokehjs_dir, kind, file) for file in files ]
         return paths
 
@@ -294,12 +170,11 @@ class BaseResources(object):
 
         return external_resources
 
-
     def _cdn_urls(self):
-        return _get_cdn_urls(self.components, self.version, self.minified)
+        return _get_cdn_urls(self.version, self.minified)
 
     def _server_urls(self):
-        return _get_server_urls(self.components, self.root_url, False if self.dev else self.minified, self.path_versioner)
+        return _get_server_urls(self.root_url, False if self.dev else self.minified, self.path_versioner)
 
     def _resolve(self, kind):
         paths = self._file_paths(kind)
@@ -314,20 +189,17 @@ class BaseResources(object):
             files = list(paths)
         elif self.mode == "cdn":
             cdn = self._cdn_urls()
-            files = list(cdn['urls'](kind))
+            files = list(cdn['urls'](self.components(kind), kind))
         elif self.mode == "server":
             server = self._server_urls()
-            files = list(server['urls'](kind))
+            files = list(server['urls'](self.components(kind), kind))
 
         return (files, raw)
 
     def _inline(self, path):
         begin = "/* BEGIN %s */" % basename(path)
-        try:
-            with open(path, 'rb') as f:
-                middle = f.read().decode("utf-8")
-        except IOError:
-            middle = ""
+        with open(path, 'rb') as f:
+            middle = f.read().decode("utf-8")
         end = "/* END %s */"  % basename(path)
         return "%s\n%s\n%s" % (begin, middle, end)
 
@@ -335,7 +207,7 @@ class JSResources(BaseResources):
     ''' The Resources class encapsulates information relating to loading or embedding Bokeh Javascript.
 
     Args:
-        mode (str) : how should Bokeh JS be included in output
+        mode (str) : How should Bokeh JS be included in output
 
             See below for descriptions of available modes
 
@@ -349,7 +221,13 @@ class JSResources(BaseResources):
 
         minified (bool, optional) : whether JavaScript should be minified or not (default: True)
 
-        root_url (str, optional) : URL and port of Bokeh Server to load resources from
+        root_url (str, optional) : URL and port of Bokeh Server to load resources from (default: None)
+
+            If ``None``, absolute URLs based on the default server configuration will
+            be generated.
+
+            ``root_url`` can also be the empty string, in which case relative URLs,
+            e.g., "static/css/bokeh.min.js", are generated.
 
             Only valid with ``'server'`` and ``'server-dev'`` modes
 
@@ -376,17 +254,15 @@ class JSResources(BaseResources):
 
     '''
 
-    def _autoload_path(self, elementid):
-        return self.root_url + "bokeh/autoload.js/%s" % elementid
+    _js_components = ["bokeh", "bokeh-widgets", "bokeh-tables", "bokeh-gl"]
+
+    # Properties --------------------------------------------------------------
 
     @property
     def js_files(self):
         files, _ = self._resolve('js')
-
         external_resources = self._collect_external_resources('__javascript__')
-        files.extend(external_resources)
-
-        return files
+        return external_resources + files
 
     @property
     def js_raw(self):
@@ -395,98 +271,12 @@ class JSResources(BaseResources):
         if self.log_level is not None:
             raw.append('Bokeh.set_log_level("%s");' % self.log_level)
 
-        custom_models = self._render_custom_models_static()
-        if custom_models is not None:
-            raw.append(custom_models)
+        if self.dev:
+            raw.append('Bokeh.settings.dev = true')
 
         return raw
 
-    _plugin_template = \
-"""
-(function outer(modules, cache, entry) {
-  if (typeof Bokeh !== "undefined") {
-    for (var name in modules) {
-      var module = modules[name];
-
-      if (typeof(module) === "string") {
-        try {
-          coffee = Bokeh.require("coffee-script")
-        } catch (e) {
-          throw new Error("Compiler requested but failed to import. Make sure bokeh-compiler(-min).js was included.")
-        }
-
-        function compile(code) {
-          var body = coffee.compile(code, {bare: true, shiftLine: true});
-          return new Function("require", "module", "exports", body);
-        }
-
-        modules[name] = [compile(module), {}];
-      }
-    }
-
-    for (var name in modules) {
-      Bokeh.require.modules[name] = modules[name];
-    }
-
-    for (var i = 0; i < entry.length; i++) {
-      Bokeh.Models.register_locations(Bokeh.require(entry[i]));
-    }
-  } else {
-    throw new Error("Cannot find Bokeh. You have to load it prior to loading plugins.");
-  }
-})({
- "custom/main":[function(require,module,exports){
-   module.exports = { %(exports)s };
- }, {}],
- %(models)s
-}, {}, ["custom/main"]);
-"""
-
-    def _render_custom_models_static(self):
-        def _escape_code(code):
-            """ Escape JS/CS source code, so that it can be embedded in a JS string.
-
-            This is based on https://github.com/joliss/js-string-escape.
-            """
-            def escape(match):
-                ch = match.group(0)
-
-                if ch == '"' or ch == "'" or ch == '\\':
-                    return '\\' + ch
-                elif ch == '\n':
-                    return '\\n'
-                elif ch == '\r':
-                    return '\\r'
-                elif ch == '\u2028':
-                    return '\\u2028'
-                elif ch == '\u2029':
-                    return '\\u2029'
-
-            return re.sub(u"""['"\\\n\r\u2028\u2029]""", escape, code)
-
-        custom_models = {}
-
-        for cls in Model.model_class_reverse_map.values():
-            impl = getattr(cls, "__implementation__", None)
-
-            if impl is not None:
-                custom_models[(cls.__module__, cls.__name__)] = impl
-
-        if not custom_models:
-            return None
-
-        exports = []
-        models = []
-
-        for (_, model_name), impl in sorted(custom_models.items(), key=lambda arg: arg[0]):
-            module_name = "custom/%s" % snakify(model_name)
-            exports.append('%s: require("%s")' % (model_name, module_name))
-            models.append('"%s": "%s"' % (module_name, _escape_code(impl)))
-
-        exports = ",\n".join(exports)
-        models = ",\n".join(models)
-
-        return self._plugin_template % dict(exports=exports, models=models)
+    # Public methods ----------------------------------------------------------
 
     def render_js(self):
         return JS_RESOURCES.render(js_raw=self.js_raw, js_files=self.js_files)
@@ -535,14 +325,15 @@ class CSSResources(BaseResources):
 
     '''
 
+    _css_components = ["bokeh", "bokeh-widgets", "bokeh-tables"]
+
+    # Properties --------------------------------------------------------------
+
     @property
     def css_files(self):
         files, _ = self._resolve('css')
-
         external_resources = self._collect_external_resources("__css__")
-        files.extend(external_resources)
-
-        return files
+        return external_resources + files
 
     @property
     def css_raw(self):
@@ -553,6 +344,8 @@ class CSSResources(BaseResources):
     def css_raw_str(self):
         return [ json.dumps(css) for css in self.css_raw ]
 
+    # Public methods ----------------------------------------------------------
+
     def render_css(self):
         return CSS_RESOURCES.render(css_raw=self.css_raw, css_files=self.css_files)
 
@@ -604,11 +397,126 @@ class Resources(JSResources, CSSResources):
 
     '''
 
+    # Public methods ----------------------------------------------------------
+
     def render(self):
         return "%s\n%s" % (self.render_css(), self.render_js())
 
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+class _SessionCoordinates(object):
+    """ Internal class used to parse kwargs for server URL, app_path, and session_id."""
+    def __init__(self, **kwargs):
+        self._url = kwargs.get('url', DEFAULT_SERVER_HTTP_URL)
+
+        if self._url is None:
+            raise ValueError("url cannot be None")
+
+        if self._url == 'default':
+            self._url = DEFAULT_SERVER_HTTP_URL
+
+        if self._url.startswith("ws"):
+            raise ValueError("url should be the http or https URL for the server, not the websocket URL")
+
+        self._url = self._url.rstrip("/")
+
+        # we lazy-generate the session_id so we can generate it server-side when appropriate
+        self._session_id = kwargs.get('session_id')
+
+    # Properties --------------------------------------------------------------
+
+    @property
+    def url(self):
+        return self._url
+
+    @property
+    def session_id(self):
+        """ Session ID derived from the kwargs provided."""
+        if self._session_id is None:
+            self._session_id = generate_session_id()
+        return self._session_id
+
+    @property
+    def session_id_allowing_none(self):
+        """ Session ID provided in kwargs, keeping it None if it hasn't been generated yet.
+
+        The purpose of this is to preserve ``None`` as long as possible... in some cases
+        we may never generate the session ID because we generate it on the server.
+        """
+        return self._session_id
+
+_DEV_PAT = re.compile(r"^(\d)+\.(\d)+\.(\d)+(dev|rc)")
+
+def _cdn_base_url():
+    return "https://cdn.pydata.org"
+
+
+def _get_cdn_urls(version=None, minified=True):
+    if version is None:
+        if settings.docs_cdn():
+            version = settings.docs_cdn()
+        else:
+            version = __version__.split('-')[0]
+
+    # check if we want minified js and css
+    _min = ".min" if minified else ""
+
+    base_url = _cdn_base_url()
+    dev_container = 'bokeh/dev'
+    rel_container = 'bokeh/release'
+
+    # check the 'dev' fingerprint
+    container = dev_container if _DEV_PAT.match(version) else rel_container
+
+    if version.endswith(('dev', 'rc')):
+        log.debug("Getting CDN URL for local dev version will not produce usable URL")
+
+    def mk_url(comp, kind):
+        return '%s/%s/%s-%s%s.%s' % (base_url, container, comp, version, _min, kind)
+
+    result = {
+        'urls'     : lambda components, kind: [ mk_url(component, kind) for component in components ],
+        'messages' : [],
+    }
+
+    if len(__version__.split('-')) > 1:
+        result['messages'].append({
+            "type" : "warn",
+            "text" : ("Requesting CDN BokehJS version '%s' from Bokeh development version '%s'. "
+                      "This configuration is unsupported and may not work!" % (version, __version__))
+        })
+
+    return result
+
+
+def _get_server_urls(root_url, minified=True, path_versioner=None):
+    _min = ".min" if minified else ""
+
+    def mk_url(comp, kind):
+        path = "%s/%s%s.%s" % (kind, comp, _min, kind)
+        if path_versioner is not None:
+            path = path_versioner(path)
+        return '%sstatic/%s' % (root_url, path)
+
+    return {
+        'urls'     : lambda components, kind: [ mk_url(component, kind) for component in components ],
+        'messages' : [],
+    }
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
+
 CDN = Resources(mode="cdn")
 
 INLINE = Resources(mode="inline")
 
-EMPTY = Resources(mode="inline", components=[], log_level=None)
+__all__ = (
+    'CDN',
+    'INLINE',
+    'Resources',
+    'JSResources',
+    'CSSResources',
+)
