@@ -7,18 +7,27 @@ from bokeh.resources import INLINE
 from bokeh.util.browser import view
 from bokeh.themes import Theme
 from bokeh.plotting import figure
+from bokeh.sampledata.iris import flowers
+from bokeh.transform import factor_cmap, factor_mark
 
-x1 = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
-y1 = [0, 8, 2, 4, 6, 9, 5, 6, 25, 28, 4]
+SPECIES = ['setosa', 'versicolor', 'virginica']
+MARKERS = ['hex', 'circle_x', 'triangle']
 
-p1 = figure(title='DARK THEMED PLOT')
-p1.scatter(x1, y1)
+p = figure(title = "Iris Morphology")
+p.xaxis.axis_label = 'Petal Length'
+p.yaxis.axis_label = 'Sepal Width'
+
+p.scatter("petal_length", "sepal_width", source=flowers, legend="species", fill_alpha=0.4, size=12,
+          marker=factor_mark('species', MARKERS, SPECIES),
+          color=factor_cmap('species', 'Category10_3', SPECIES))
+
+p.legend.background_fill_color = "#3f3f3f"
 
 theme = Theme(json={
     'attrs': {
         'Figure': {
-            'background_fill_color': '#2F2F2F',
-            'border_fill_color': '#2F2F2F',
+            'background_fill_color': '#3f3f3f',
+            'border_fill_color': '#3f3f3f',
             'outline_line_color': '#444444'
             },
         'Axis': {
@@ -29,21 +38,21 @@ theme = Theme(json={
             'minor_tick_line_color': "white",
             'minor_tick_line_color': "white"
             },
+        'Legend': {
+            'background_fill_color': '#3f3f3f',
+            'label_text_color': "white",
+        },
         'Grid': {
             'grid_line_dash': [6, 4],
             'grid_line_alpha': .3
             },
-        'Circle': {
-            'fill_color': 'lightblue',
-            'size': 10,
-            },
         'Title': {
             'text_color': "white"
             }
         }
     })
 
-script, div = components(p1, theme=theme)
+script, div = components(p, theme=theme)
 
 template = Template('''<!DOCTYPE html>
 <html lang="en">
@@ -54,7 +63,7 @@ template = Template('''<!DOCTYPE html>
         {{ script }}
         <style>
             body {
-                background: #2F2F2F;
+                background: #3f3f3f;
             }
 
             .embed-wrapper {
