@@ -8,8 +8,8 @@ y = np.linspace(0, 10, N)
 xx, yy = np.meshgrid(x, y)
 d = np.sin(xx)*np.cos(yy)
 
-p = figure(x_range=(0, 10), y_range=(0, 10),
-           tooltips=[("x", "$x"), ("y", "$y"), ("value", "@image")])
+p = figure(tooltips=[("x", "$x"), ("y", "$y"), ("value", "@image")])
+p.x_range.range_padding = p.y_range.range_padding = 0
 
 # must give a vector of image data for image parameter
 p.image(image=[d], x=0, y=0, dw=10, dh=10, palette="Spectral11")
