@@ -1,5 +1,4 @@
 from bokeh.io import output_file, show
-from bokeh.layouts import widgetbox
 from bokeh.models.widgets import CheckboxButtonGroup
 
 output_file("checkbox_button_group.html")
@@ -7,4 +6,4 @@ output_file("checkbox_button_group.html")
 checkbox_button_group = CheckboxButtonGroup(
         labels=["Option 1", "Option 2", "Option 3"], active=[0, 1])
 
-show(widgetbox(checkbox_button_group))
+show(checkbox_button_group)
