@@ -11,16 +11,16 @@ invisible_line = p.line([1, 2, 3], [2, 1, 2], line_color="pink")
 box = BoxAnnotation(left=1.5, right=2.5, fill_color='green', fill_alpha=0.1)
 p.add_layout(box)
 
-# We write coffeescript to link toggle with visible property of box and line
+# We write JavaScript to link toggle with visible property of box and line
 code = '''\
 object.visible = toggle.active
 '''
 
-callback1 = CustomJS.from_coffeescript(code=code, args={})
+callback1 = CustomJS(code=code, args={})
 toggle1 = Toggle(label="Green Box", button_type="success", callback=callback1)
 callback1.args = {'toggle': toggle1, 'object': box}
 
-callback2 = CustomJS.from_coffeescript(code=code, args={})
+callback2 = CustomJS(code=code, args={})
 toggle2 = Toggle(label="Pink Line", button_type="success", callback=callback2)
 callback2.args = {'toggle': toggle2, 'object': invisible_line}
 
